﻿

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Devices.Sensors;

namespace Compass71
{
    public partial class MainPage : PhoneApplicationPage
    {
        Compass compass = new Compass();
        RotateTransform transform = new RotateTransform();
        double TrueHeading;
        double ReciprocalHeading;
        double HeadingAccuracy;
        bool Alphabetic = false; //alpha output setting, default is false
        bool Calibrating = false;

         // Constructor
        public MainPage()
        {
            try
            {
                InitializeComponent();
                compass.CurrentValueChanged += new EventHandler<SensorReadingEventArgs<CompassReading>>(compass_CurrentValueChanged);
                compass.Calibrate += new EventHandler<CalibrationEventArgs>(compass_Calibrate);

                //Check visibility theme and respond appropriately
                //Fix added to address issue on first app submission
                Visibility isLight = (Visibility)Resources["PhoneLightThemeVisibility"]; // for light theme
                if (isLight == System.Windows.Visibility.Visible) //light theme enabled
                {

                    //constructor technique
                    SolidColorBrush scb = new SolidColorBrush(Colors.Black);
                    SolidColorBrush scb2 = new SolidColorBrush(Colors.Red);

                    RecipLabelTextBlock.Foreground = scb;
                    HeadingLabelTextBlock.Foreground = scb;

                    RecipTextBlock.Foreground = scb2;
                    HeadingTextBlock.Foreground = scb2;

                    LockButton.Foreground = scb;
                    UnlockButton.Foreground = scb;
                    AlphaButton.Foreground = scb;
                    NumericButton.Foreground = scb;
                }
                else //dark color scheme is selected--set text colors appropriately
                {
                    //constructor technique
                    SolidColorBrush scb = new SolidColorBrush(Colors.DarkGray);
                    SolidColorBrush scb2 = new SolidColorBrush(Colors.LightGray);

                    RecipLabelTextBlock.Foreground = scb;
                    HeadingLabelTextBlock.Foreground = scb;

                    RecipTextBlock.Foreground = scb2;
                    HeadingTextBlock.Foreground = scb2;

                    LockButton.Foreground = scb;
                    UnlockButton.Foreground = scb;
                    AlphaButton.Foreground = scb;
                    NumericButton.Foreground = scb;
                }

                compass.TimeBetweenUpdates = TimeSpan.FromMilliseconds(400);  // Must be multiple of 20
                compass.Start();
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButton.OK);
            } 
        }

        ~MainPage()
        {
            try
            {
                compass.Stop(); //Stop the compass on shutdown
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButton.OK);
            }
        }

        void compass_Calibrate(object sender, CalibrationEventArgs e)
        {
            try
            {
                Dispatcher.BeginInvoke(() => { calibrationStackPanel.Visibility = Visibility.Visible; });
                Calibrating = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButton.OK);
            }
        }

        private void calibrationButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                calibrationStackPanel.Visibility = Visibility.Collapsed;
                Calibrating = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButton.OK);
            }
        }

        void compass_CurrentValueChanged(object sender, SensorReadingEventArgs<CompassReading> e)
        {
            // Note that this event handler is called from a background thread
            // and therefore does not have access to the UI thread. To update 
            // the UI from this handler, use Dispatcher.BeginInvoke() as shown.
            // Dispatcher.BeginInvoke(() => { statusTextBlock.Text = "in CurrentValueChanged"; });

            try
            {
                if (compass.IsDataValid)
                {
                    Dispatcher.BeginInvoke(() =>
                   {

                       HeadingAccuracy = e.SensorReading.HeadingAccuracy;

                       if (!Calibrating)
                       {

                           TrueHeading = e.SensorReading.TrueHeading;
                           if ((180 <= TrueHeading) && (TrueHeading <= 360))
                               ReciprocalHeading = TrueHeading - 180;
                           else
                               ReciprocalHeading = TrueHeading + 180;

                           CompassFace.RenderTransformOrigin = new Point(0.5, 0.5);
                           EllipseGlass.RenderTransformOrigin = new Point(0.5, 0.5); //new

 
                           transform.Angle = 360 - TrueHeading;
                           CompassFace.RenderTransform = transform;
                           EllipseGlass.RenderTransform = transform; // new

                           if (!Alphabetic) //render numeric heading
                           {
                               HeadingTextBlock.Text = TrueHeading.ToString();
                               RecipTextBlock.Text = ReciprocalHeading.ToString();
                           }
                           else // render alpha heading
                           {
                               if (((337 <= TrueHeading) && (TrueHeading < 360)) ||
                                  ((0 <= TrueHeading) && (TrueHeading < 22)))
                               {
                                   HeadingTextBlock.Text = "N";
                                   RecipTextBlock.Text = "S";
                               }
                               else if ((22 <= TrueHeading) && (TrueHeading < 67))
                               {
                                   HeadingTextBlock.Text = "NE";
                                   RecipTextBlock.Text = "SW";
                               }
                               else if ((67 <= TrueHeading) && (TrueHeading < 112))
                               {
                                   HeadingTextBlock.Text = "E";
                                   RecipTextBlock.Text = "W";
                               }
                               else if ((112 <= TrueHeading) && (TrueHeading < 152))
                               {
                                   HeadingTextBlock.Text = "SE";
                                   RecipTextBlock.Text = "NW";
                               }
                               else if ((152 <= TrueHeading) && (TrueHeading < 202))
                               {
                                   HeadingTextBlock.Text = "S";
                                   RecipTextBlock.Text = "N";
                               }
                               else if ((202 <= TrueHeading) && (TrueHeading < 247))
                               {
                                   HeadingTextBlock.Text = "SW";
                                   RecipTextBlock.Text = "NE";
                               }
                               else if ((247 <= TrueHeading) && (TrueHeading < 292))
                               {
                                   HeadingTextBlock.Text = "W";
                                   RecipTextBlock.Text = "E";
                               }
                               else if ((292 <= TrueHeading) && (TrueHeading < 337))
                               {
                                   HeadingTextBlock.Text = "NW";
                                   RecipTextBlock.Text = "SE";
                               }

                           }

                       } // endif !Calibrating

                       else
                       {
                           if (HeadingAccuracy <= 10)
                           {
                               CalibrationTextBlock.Foreground = new SolidColorBrush(Colors.Green);
                               CalibrationTextBlock.Text = "Complete!";
                           }
                           else
                           {
                               CalibrationTextBlock.Foreground = new SolidColorBrush(Colors.Red);
                               CalibrationTextBlock.Text = HeadingAccuracy.ToString("0.0");
                           }
                       }
                   });
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButton.OK);
            }
        }

        private void LockButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                compass.Stop();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButton.OK);
            }
        }

        private void UnlockButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                compass.Start();
                compass.TimeBetweenUpdates = TimeSpan.FromMilliseconds(400);  // Must be multiple of 20
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButton.OK);
            }
        }


        // Numeric output (default)
        private void NumericButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Alphabetic = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButton.OK);
            }
        }

        // Letter (or alphabetic) output
        // N, NNW, WNW, W, WSW, SSW, S, SSE, ESE, E, ENE, NNE
        private void AlphaButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Alphabetic = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButton.OK);
            }
        }

     }
}